@@ -1,15 +1,26 @@
 #!/bin/bash
 # Usage: 
-# for p in 5.6.2 5.8.9d 5.10.1 5.11.2; do make -q clean >/dev/null; perl$p Makefile.PL; t/testc.sh -q 26; t/testcc.sh -q 26; done
+# for p in 5.6.2 5.8.9d 5.10.1 5.11.2; do make -q clean >/dev/null; perl$p Makefile.PL; t/testplc.sh -q -c; done
 # use the actual perl from the Makefile (perld, perl5.10.0, perl5.8.8, perl5.11.0, ...)
+
+function help {
+  echo "t/testplc.sh [OPTIONS] [1-26]"
+  echo " -s                 skip B:Debug, roundtrips and options"
+  echo " -c                 continue on errors"
+  echo " -q                 quiet"
+  echo " -h                 help"
+  echo "t/testplc.sh -q -s -c <=> perl -Mblib t/bytecode.t"
+  echo "Without arguments try all 26 tests. Without Option -Ox try all three optimizations."
+}
+
 PERL=`grep "^PERL =" Makefile|cut -c8-`
 PERL=${PERL:-perl}
 #PERL=perl5.11.0
 VERS=`echo $PERL|sed -e's,.*perl,,' -e's,.exe$,,'`
 D="`$PERL -e'print (($] < 5.007) ? q(256) : q(v))'`"
 # test what? core or our module?
-#Mblib="`$PERL -e'print (($] < 5.009005) ? q() : q(-Mblib))'`"
-Mblib="-Mblib" # test this module
+#Mblib="`$PERL -e'print (($] < 5.008) ? q() : q(-Mblib))'`"
+test -z "$CORE" && Mblib="-Mblib" # test this module
 OCMD="$PERL $Mblib -MO=Bytecode,"
 QOCMD="$PERL $Mblib -MO=-qq,Bytecode,"
 ICMD="$PERL $Mblib -MByteLoader"
@@ -19,14 +30,14 @@ if [ -z $Mblib ]; then VERS="${VERS}_global"; fi
 function pass {
     #echo -n "$1 PASS "
     echo -e -n "\e[1;32mPASS \e[0;0m"
-    shift
+    #shift
     echo $*
     echo
 }
 function fail {
     #echo -n "$1 FAIL "
     echo -e -n "\e[1;31mFAIL \e[0;0m"
-    shift
+    #shift
     echo $*
     echo
 }
@@ -35,7 +46,7 @@ function bcall {
     o=$1
     opt=${2:-s}
     ext=${3:-plc}
-    echo ${QOCMD}-$opt,-o${o}${opt}_${VERS}.${ext} ${o}.pl
+    [ -n "$Q" ] || echo ${QOCMD}-$opt,-o${o}${opt}_${VERS}.${ext} ${o}.pl
     ${QOCMD}-$opt,-o${o}${opt}_${VERS}.${ext} ${o}.pl
 }
 function btest {
@@ -50,9 +61,10 @@ function btest {
       echo "$2" > ${o}.pl
   fi
   #bcall ${o} O6
-  rm ${o}s_${VERS}.disasm ${o}_s_${VERS}.plc 2>/dev/null
+  rm ${o}_s_${VERS}.plc 2>/dev/null
   
-  if [ -n "$Mblib" ]; then 
+  if [ -n "$Mblib" -a -z "$SKIP" ]; then 
+    rm ${o}s_${VERS}.disasm ${o}_s_${VERS}.concise ${o}_s_${VERS}.dbg 2>/dev/null
     bcall ${o} s
     [ -n "$Q" ] || echo $PERL $Mblib script/disassemble ${o}s_${VERS}.plc \> ${o}s_${VERS}.disasm
     $PERL $Mblib script/disassemble ${o}s_${VERS}.plc > ${o}s_${VERS}.disasm
@@ -73,15 +85,17 @@ function btest {
 
     bcall ${o} k
     $PERL $Mblib script/disassemble ${o}k_${VERS}.plc > ${o}k_${VERS}.disasm
-    [ -n "$Q" ] || echo $PERL $Mblib -MO=Debug,-exec ${o}.pl -o ${o}_${VERS}.dbg
-    $PERL $Mblib -MO=Debug,-exec ${o}.pl > ${o}_${VERS}.dbg
+    [ -n "$Q" ] || echo $PERL $Mblib -MO=${qq}Debug,-exec ${o}.pl -o ${o}_${VERS}.dbg
+    $PERL $Mblib -MO=${qq}Debug,-exec ${o}.pl > ${o}_${VERS}.dbg
   fi
-  # 5.8 has a bad concise
-  [ -n "$Q" ] || echo $PERL $Mblib -MO=Concise,-exec ${o}.pl -o ${o}_${VERS}.concise
-  $PERL $Mblib -MO=Concise,-exec ${o}.pl > ${o}_${VERS}.concise
-  if [ -n "$Mblib" ]; then 
-    #bcall ${o} TI
-    bcall ${o} H
+  if [ -z "$SKIP" ]; then
+    # 5.8 has a bad concise
+    [ -n "$Q" ] || echo $PERL $Mblib -MO=${qq}Concise,-exec ${o}.pl -o ${o}_${VERS}.concise
+    $PERL $Mblib -MO=${qq}Concise,-exec ${o}.pl > ${o}_${VERS}.concise
+    if [ -n "$Mblib" ]; then 
+      #bcall ${o} TI
+      bcall ${o} H
+    fi
   fi
   if [ -n "$Mblib" ]; then
     # -s ("scan") should be the new default
@@ -95,10 +109,11 @@ function btest {
   [ -n "$Q" ] || echo ${ICMD} ${o}.plc
   res=$(${ICMD} ${o}.plc)
   if [ "X$res" = "X${result[$n]}" ]; then
-      test "X$res" = "X${result[$n]}" && pass "./${o}_o1" "=> '$res'"
+      test "X$res" = "X${result[$n]}" && pass "./${o}.plc" "=> '$res'"
   else
-      fail "./${o}_o1" "=> '$res' Expected: '${result[$n]}'"
-      ( [ -n "$Q" ] || echo ${ICMD} -D$D ${o}.plc; ${ICMD} -D$D ${o}.plc || exit )
+      fail "./${o}.plc" "'$str' => '$res' Expected: '${result[$n]}'"
+      [ -n "$Q" ] || (echo ${ICMD} -D$D ${o}.plc; ${ICMD} -D$D ${o}.plc)
+      test -z $CONT && exit
   fi
 }
 
@@ -113,7 +128,7 @@ tests[3]='$_ = "xyxyx"; %j=(1,2); s/x/$j{print("z")}/ge; print $_'
 result[3]='zzz2y2y2';
 tests[4]='$_ = "xyxyx"; %j=(1,2); s/x/$j{print("z")}/g; print $_'
 result[4]='z2y2y2';
-tests[5]='split /a/,"bananarama"; print @_'
+tests[5]='print split /a/,"bananarama"'
 result[5]='bnnrm';
 tests[6]="{package P; sub x {print 'ya'} x}"
 result[6]='ya';
@@ -173,15 +188,30 @@ result[25]="0 1 2 3`$PERL -e'print (($] < 5.007) ? q( 4 5) : q())'` 4321";
 tests[26]='sub a:lvalue{my $a=26; ${\(bless \$a)}}sub b:lvalue{${\shift}}; print ${a(b)}';
 result[26]="26";
 
-if [ "$1" = "-q" ]; then Q=1; shift; fi
-if [ "$1" = "-c" ]; then CONT=1; shift; fi
 
-if [ -z "$QUIET" ]; then
+while getopts "qscCh" opt
+do
+  if [ "$opt" = "q" ]; then
+      Q=1
+      OCMD="$QOCMD"
+      qq="-qq,"
+      if [ "$VERS" = "5.6.2" ]; then QOCMD=$OCMD; qq=""; fi
+  fi
+  if [ "$opt" = "s" ]; then SKIP=1; fi
+  if [ "$opt" = "c" ]; then CONT=1; shift; fi
+  if [ "$opt" = "C" ]; then CORE=1; shift; fi
+  if [ "$opt" = "h" ]; then help; exit; fi
+done
+
+if [ -z "$Q" ]; then
     make
 else
-    make -q >/dev/null
+    make --silent >/dev/null
 fi
 
+# need to shift the options
+while [ -n "$1" -a "${1:0:1}" = "-" ]; do shift; done
+
 if [ -n "$1" ]; then
   while [ -n "$1" ]; do
     btest $1
